import json
from config import data_path_logs, data_path_captcha, data_path_done, annotation_threshold, true_threshold, \
    false_threshold
import os
import shutil

# Uložení jsonu
def save_results_to_file(results, filename):
    with open(filename, 'w') as file:
        json.dump(results, file)


#Úprava dat v json logu
def process_json_data(category):
    with open(os.path.join(data_path_logs, 'results.json'), 'r') as file:
        json_data = json.load(file)
    # Vybere záznamy pro danou složku/dataset
    category_records = [record for record in json_data['results'] if record.get('folder_name') == category]

    # Uložíme zvlášť
    save_results_to_file(category_records, os.path.join(data_path_logs, f'{category}_category_results.json'))

    # Smažeme z původního jsonu
    json_data['results'] = [record for record in json_data['results'] if record.get('folder_name') != category]

    # Uložíme původní json
    save_results_to_file(json_data, os.path.join(data_path_logs, 'results.json'))


# Filtrace úspěšných CAPTCHA
def filter_success_results():
    with open(os.path.join(data_path_logs, 'results.json'), 'r') as file:
        data = json.load(file)
        # Vyber splněné CAPTCHA
        success_results = [result for result in data['results'] if result['status'] == 'success']
    return success_results


# Zpracujeme výsledky
def process_captcha_results(success_results):
    result_dict = {}
    # Projdeme každý záznam v JSON
    for item in success_results:
        folder_name = item.get('folder_name')

        if folder_name:
            unknown_images = item.get('unknown_images', [])
            selected_images = item.get('selected_images', [])

            # Inicializujeme slovníky pro 'unknown_true' a 'unknown_false' pro tento 'folder_name'
            if folder_name not in result_dict:
                result_dict[folder_name] = {'unknown_tested': {}, 'unknown_true': {}, 'unknown_false': {}}

            for image in unknown_images:
                # Připočteme si výskyt obrázku
                if image in result_dict[folder_name]['unknown_tested']:
                    result_dict[folder_name]['unknown_tested'][image] += 1
                else:
                    result_dict[folder_name]['unknown_tested'][image] = 1
                # Pokud je obrázek v 'selected_images', přidáme ho do 'unknown_true'
                if image in selected_images:
                    if image in result_dict[folder_name]['unknown_true']:
                        result_dict[folder_name]['unknown_true'][image] += 1
                    else:
                        result_dict[folder_name]['unknown_true'][image] = 1
                # Ostatní jsou 'unknown_false'
                else:
                    if image in result_dict[folder_name]['unknown_false']:
                        result_dict[folder_name]['unknown_false'][image] += 1
                    else:
                        result_dict[folder_name]['unknown_false'][image] = 1
    return result_dict


# Zpracování result_dict
def process_result_dict(result_dict):
    done_folders = []
    for folder_name, values in result_dict.items():
        # Pro každou složku vybereme počet předložení každého obrázku
        unknown_tested = values['unknown_tested']
        # Najdeme ten s nejmenším počtem předložení a porovnáme s thresholdem
        min_value = min(unknown_tested.values())
        if min_value < annotation_threshold:
            print(
                f"Složka {folder_name} obsahuje obrázky, které nesplňují minimální počet předložení ke kontrole. Složka bude ponechána v CAPTCHA")
        else:
            # Když je OK, můžeme složku poslat dále ke zpracování
            done_folders.append(folder_name)

    return (done_folders)


# Analýza výsledků pro jednotlivé datasety/složky
def analyze_category_results(result_dict, done_folders):
    for folder_name, values in {key: value for key, value in result_dict.items() if key in done_folders}.items():
        # Inicializace
        true_list = []
        false_list = []
        unknown_list = []
        result_detail = {}
        # Dopočet ratia a rozřazení obrázků
        for image, tested_value in values['unknown_tested'].items():
            if image in values['unknown_true']:
                true_value = values['unknown_true'][image]
                ratio = true_value / tested_value
                result_detail[image] = ratio
                # Kontrola thresholdu
                if ratio >= true_threshold:
                    true_list.append(image)
                elif ratio <= false_threshold:
                    false_list.append(image)
                else:
                    unknown_list.append(image)
            else:
                result_detail[image] = 0
                false_list.append(image)

        # Volání dalších funkcí
        classify_and_move_images(folder_name, true_list, false_list)
        process_json_data(folder_name)
        # Ukládáme spočítanou míru ohodnocení
        save_results_to_file(result_detail, os.path.join(data_path_logs, f'{folder_name}_results_ratio.json'))


# Roztřídění obrázků do složek na základě rozdělení
def classify_and_move_images(category, true_list, false_list):
    captcha_category_path = os.path.join(os.path.dirname(os.getcwd()), data_path_captcha, category)
    done_folder_path = os.path.join(data_path_done, category)
    # Příprava prázdných složek
    os.makedirs(done_folder_path)
    for subfolder in ['true', 'false', 'unknown']:
        subfolder_path = os.path.join(done_folder_path, subfolder)
        os.makedirs(subfolder_path)

    unknown_captcha_path = os.path.join(captcha_category_path, 'unknown')
    for file_name in os.listdir(unknown_captcha_path):
        file_path = os.path.join(unknown_captcha_path, file_name)

        # Rozdělení obrázků do složek
        if file_name in true_list:
            destination_folder = os.path.join(done_folder_path, 'true')
        elif file_name in false_list:
            destination_folder = os.path.join(done_folder_path, 'false')
        else:
            destination_folder = os.path.join(done_folder_path, 'unknown')

        shutil.move(file_path, destination_folder)

    # Přesun dat ze složky true
    true_captcha_path = os.path.join(captcha_category_path, 'true')
    for file_name in os.listdir(true_captcha_path):
        file_path = os.path.join(true_captcha_path, file_name)
        true_destination_path = os.path.join(done_folder_path, 'true')
        shutil.move(file_path, true_destination_path)
    # Snazání původní složky
    shutil.rmtree(captcha_category_path)


# Spouštění jednotlivých funkcí pro vyhodnocení
def process_data_evaluation():
    successfull_data = filter_success_results()
    result_dict = process_captcha_results(successfull_data)
    anotated_folders = process_result_dict(result_dict)
    analyze_category_results(result_dict, anotated_folders)


# Spuštění vyhodnocení dat
if __name__ == '__main__':
    process_data_evaluation()
