import os

from config import data_path_raw, data_path_processed, required_true_pictures, required_unknown_pictures, \
    data_path_captcha, image_size
from PIL import Image
import shutil


# Vybere nejvíce zanořený element cesty
def get_most_nested_element(path):
    # Split the path into components
    elements = path.split(os.path.sep)

    # Iterate through the components in reverse order
    for element in reversed(elements):
        # If the component is not empty, return it (this is the most nested folder)
        return element


# Připraví obrázky pro CAPTCHA
def prepare_picture(output_folder, file, file_name):
    try:
        # Zvolíme veliost
        size = image_size
        with Image.open(file) as img:
            img.thumbnail(size, Image.LANCZOS)
            # Bílý podklad
            new_img = Image.new("RGB", size, "white")
            left = (size[0] - img.width) // 2
            top = (size[1] - img.height) // 2
            right = left + img.width
            bottom = top + img.height
            # Vloží obrázek doprostřed bíleho podkladu
            new_img.paste(img, (left, top, right, bottom))
            new_img.save(os.path.join(output_folder, file_name + ".jpg"))
    except:
        raise Exception(
            f"Chyba v přípravě obrázku {get_most_nested_element(file)} ze složky folder {get_most_nested_element(output_folder)}")


# Kontrola struktruy složek
def check_structure(folder_path):
    # Kontorla, zda existují složky true a unknown
    folders = [item for item in os.listdir(folder_path) if os.path.isdir(os.path.join(folder_path, item))]
    if not ("true" in folders and "unknown" in folders):
        raise Exception(f"Požadované složky nejsou nejsou ve složce {get_most_nested_element(folder_path)}.")

    true_folder_path = os.path.join(folder_path, "true")
    unknown_folder_path = os.path.join(folder_path, "unknown")

    # Kontrola, jestli složky true a unknown obsahují požadovaný počet obrázků
    true_count = len(
        [item for item in os.listdir(true_folder_path) if os.path.isfile(os.path.join(true_folder_path, item))])
    unknown_count = len(
        [item for item in os.listdir(unknown_folder_path) if os.path.isfile(os.path.join(unknown_folder_path, item))])

    if true_count < required_true_pictures:
        raise Exception(
            f"Není splňen minimální počet obrázku ve složce true. Požadováno: {true_count} Nalezeno: {required_true_pictures}")
    if unknown_count < required_unknown_pictures:
        raise Exception(
            f"Není splňen minimální počet obrázku ve složce unknown. Požadováno: {unknown_count} Nalezeno: {required_unknown_pictures}")


# Příprava obrázků ze složky 01_raw
def preprocess_raw_folders():
    for folder_name in [item for item in os.listdir(data_path_raw) if os.path.isdir(os.path.join(data_path_raw, item))]:
        raw_folder_path = os.path.join(data_path_raw, folder_name)
        raw_folder_path_true = os.path.join(raw_folder_path, "true")
        raw_folder_path_unknown = os.path.join(raw_folder_path, "unknown")
        # Voláme kontrolu struktruy složek
        check_structure(raw_folder_path)

        # Kontrola, zda už složku v připravených nemáme
        processed_folder_path = os.path.join(data_path_processed, folder_name)
        if os.path.exists(processed_folder_path):
            raise Exception(f"Složka {folder_name} již existuje v adresáři připravených složek.")

        os.makedirs(processed_folder_path)

        processed_folder_path_true = os.path.join(processed_folder_path, "true")
        processed_folder_path_unknown = os.path.join(processed_folder_path, "unknown")

        os.makedirs(processed_folder_path_true)
        os.makedirs(processed_folder_path_unknown)

        # Připravuejem obrázky v test složce
        for index, file in enumerate(os.listdir(raw_folder_path_true)):
            file_path = os.path.join(raw_folder_path_true, file)
            prepare_picture(processed_folder_path_true, file_path, "t" + str(index))
        # Připravuejem obrázky v unknown složce
        for index, file in enumerate(os.listdir(raw_folder_path_unknown)):
            file_path = os.path.join(raw_folder_path_unknown, file)
            prepare_picture(processed_folder_path_unknown, file_path, "u" + str(index))


# Přesun připravené složky do úložiště CAPTCHA
def move_processed_folders_to_captcha():
    for folder in os.listdir(data_path_processed):
        captcha_path = os.path.join(os.path.dirname(os.getcwd()), data_path_captcha)
        shutil.move(os.path.join(data_path_processed, folder), captcha_path)


# Spuštění přípravy dat s přesunem
if __name__ == "__main__":
    preprocess_raw_folders()
    move_processed_folders_to_captcha()
